import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.lang.*;
import java.util.*;
import javax.sound.sampled.*;
import java.net.URL;
import java.io.File;
import java.io.IOException;

public enum SoundEffect {
    PING("sounds/17__anton__glass-a-pp.wav",7),

    BREAK("sounds/glass-break.wav"),

    BUZZ("sounds/bumblebee.wav"),
    BELL("sounds/small_ring.wav"),
    FAIL("sounds/price-is-right-fail.wav");

    // Nested class for specifying volume
    public static enum Volume {
        MUTE, LOW, MEDIUM, HIGH
    }
    
    public static Volume volume = Volume.LOW;

    // Each sound effect has its own clip, loaded with its own sound file.
    private Clip[] clip;
    private int NumClips = 1;
    private int Index = 0;
    
    SoundEffect(String soundFileName)
    {
        this(soundFileName,1);
    }
    
    // Constructor to construct each element of the enum with its own sound file.
    SoundEffect(String soundFileName,int numClips) {

        NumClips = numClips;
        
        try {
            // Use URL (instead of File) to read from disk and JAR.
            URL url = this.getClass().getClassLoader().getResource(soundFileName);
            // Set up an audio input stream piped from the sound file.
            AudioInputStream []audioInputStream = null;
            audioInputStream = new AudioInputStream[NumClips];
            
            if(null == url)
            {
                File f = new File(soundFileName);
                for(int i = 0 ; i < NumClips; i++)
                {
                    audioInputStream[i] = AudioSystem.getAudioInputStream(f);
                }
            }
            else
            {
                for(int i = 0 ; i < NumClips; i++)
                {
                    audioInputStream[i] = AudioSystem.getAudioInputStream(url); 
                }
            }
            // Get a clip resource.
            clip = new Clip[numClips];
            
            for(int i = 0 ; i < numClips ; i++)
            {
                clip[i] = AudioSystem.getClip();
                // Open audio clip and load samples from the audio input stream.
                clip[i].open(audioInputStream[i]);
            }
        } catch (UnsupportedAudioFileException e) {
            e.printStackTrace();
            NumClips = 0;
        } catch (IOException e) {
            e.printStackTrace();
            NumClips = 0;
        } catch (LineUnavailableException e) {
            e.printStackTrace();
            NumClips = 0;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            NumClips = 0;
        }

    }
    // Play or Re-play the sound e/ffect from the beginning, by rewinding.
    public void play() {
        
        Index = (Index + 1) % NumClips;
        clip[Index].stop();
        clip[Index].setFramePosition(0);
        
        if (volume != Volume.MUTE) {
            //if (clip.isRunning())
            //    clip.stop();   // Stop the player if it is still running
            //clip.setFramePosition(0);
            clip[Index].start();     // Start playing
        }
    }

    /* Will have to give a pause instance
    public void pause()
    {
        if (clip.isRunning())
            clip.stop();
    }
    */
   
    public void stop()
    {
        for(int i = 0; i < NumClips; i++)
        {
            if (clip[i].isRunning())
                clip[i].stop();

                clip[i].setFramePosition(0);
        }
    }

    public void playIfNotPlaying()
    {
        Index = (Index + 1) % NumClips;
        if (volume != Volume.MUTE) {
            if (clip[Index].isRunning())
                return;   // Stop the player if it is still running
            clip[Index].setFramePosition(0); // rewind to the beginning
            clip[Index].start();     // Start playing
        }
    }

    // Optional static method to pre-load all the sound files.
    static void init() {
        values(); // calls the constructor for all the elements
    }
}